
/* R-Dev-Ino Mapper 
   - uses I2C or Serial communication interfaces
   - uses a Nokia Color LCD
   - I2C master or slave mode
   - uses one Ping sensor
   - uses 2 Sharp Long Range sensors
   - uses 1 Thermopile Array sensor
*/ 


// R-Dev-Ino board pinout:
//
// Ext  RDI Funct Arduino  ATmega168      Arduino Funct RDI External
//                       +-----\/----+
//          Reset       1| PC6   PC5 |28  D19 A5  SCL
//          Rx    D0    2| PD0   PC4 |27  D18 A4  SDA
//          Tx    D1    3| PD1   PC3 |26  D17 A3        
//      Red Int0  D2    4| PD2   PC2 |25  D16 A2        
//      Grn Int1  D3    5| PD3   PC1 |24  D15 A1        
//      Blu       D4    6| PD4   PC0 |23  D14 A0        
//                      7| VCC   GND |22  
//                      8| GND  AREF |21  
//          Xtal        9| PB6  AVCC |20  
//          Xtal       10| PB7   PB5 |19  D13      SCK  SCK
//      Pan OC0B  D5   11| PD5   PB4 |18  D12      MISO 
//      Tlt OC0A  D6   12| PD6   PB3 |17  D11 OC2A MOSI SDA      
//      Sw1       D7   13| PD7   PB2 |16  D10 OC1B      CSE
//      Sw2       D8   14| PB0   PB1 |15  D 9 OC1A      Rst
//                       +-----------+
//

#include <Nokia_lcd.h>
#include <Servo.h>
#include <Wire.h>
//#include <AVRcam.h>

// I2C slaves addresses
#define Compass         0x21
#define Speech          50>>1
#define MotorController 60>>1
#define ServoController 70>>1
#define LedBar          80>>1
#define Mapper          90>>1
#define Brain           100>>1
#define Thermo          208>>1

// I2C commands
#define Forward     1 //MotorController
#define Reverse     2 //MotorController
#define Left        3 //MotorController/ServoController
#define Right       4 //MotorController/ServoController
#define StopAll     5 //MotorController
#define GoFwd       6 //MotorController
#define GoBwd       7 //MotorController
#define Err         8 //MotorController

#define Up          1 //ServoController
#define Down        2 //ServoController

#define Swaist     11 //ServoController
#define SleftSh    12 //ServoController
#define SrightSh   13 //ServoController
#define SleftArm   14 //ServoController
#define SrightArm  15 //ServoController
#define Sscan      16 //ServoController
#define SleftHand  17 //ServoController
#define SrightHand 18 //ServoController
#define SpanHead   19 //ServoController
#define StiltHead  20 //ServoController
#define StoreMove  21 //ServoController
#define PickUp     22 //ServoController
#define Release    23 //ServoController
#define Wave       24 //ServoController

#define LCD1       31 //Speech
#define LCD2       32 //Speech
#define Heading    33 //Speech
#define Node       34 //Speech
#define Room       35 //Speech
#define Ping       36 //Speech
#define Sharp      37 //Speech
#define Angle      38 //Speech
#define Compass    39 //Speech
#define Temp       40 //Speech
#define Cluster    41 //Speech
#define FoundFire  42 //Speech
#define NoFire     43 //Speech

#define Scan       61 //Mapper
#define Fire       62 //Mapper
#define Fetch      63 //Mapper
#define Dance      64 //Mapper

#define Done       99 //Mapper

//IN-OUTs
#define PingPin  14     // digital pin 14
#define SrightPin 1     // analog pin 1
#define SleftPin  2     // analog pin 2
#define SscanPin  3     // analog pin 3

// Nokia LCD pins:
//#define RedPin    2     // digital pin 2
//#define BluePin   3     // digital pin 3
//#define GreenPin  4     // digital pin 4
//#define Sw2Pin    5     // digital pin 5
//#define Sw1Pin    6     // digital pin 6
//#define RESET     9
//#define CS       10
//#define DATA     11
//#define CLK      13

//Servos
#define PanPin    7     // digital pin 7
#define TiltPin   6     // digital pin 6

//Variables
byte PingDistance = 0;
byte USValue;
byte LIRvalue;
byte RIRvalue;
byte val = 0;
byte temperature[] = {0,0,0,0,0,0,0,0,0}; // Array to hold temperature data
byte hotObject = 0;  // fire temperature
byte hottest[2]={0,0};
char text [50];
char byteString[4];
byte x=0;
byte y=0;
byte a=0;
byte odometer=1;                   // the metric odometer
byte headingData[2];
int j, deg, frac;
boolean room=0;
//camera stuff
int dx=0, dy=0;
byte xMid = 144/2;
byte yMid = 88/2;
byte num = 0;

//Map data
/*
// byte SmallMap[10][10];
// byte KitchenMap[33][12]
             // x axis index  0   1   2   3   4   5   6   7   8   9  10  11  12  13  14  15  16  17     // y axis index
//    LivingRoomMap[y][x]   0-20, 40, 60, 80,100,120,140,160,180,200,220,240,260,280,300,320,340,360 cm // cm
byte LivingRoomMap[33][18]={{100,100,100,100,100,100,100,100,100,100,255,255,255,255,255,200,200,200},  // 20   0
                            {  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,210,200,200,200,200,200,200,200},  // 40   1
                            {  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,200,255,255,255,255,255,200,200},  // 60   2
                            {  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,200,255,255,255,255,255,200,200},  // 80   3
                            { 40,  0,  0,  0,  0,  0,  0,  0,  0,  0,200,255,255,255,255,255,200,200},  //100   4
                            {100,  0,  0,  0,  0,  0,  0,  0,  0,  0,200,255,255,255,255,255,200,200},  //120   5
                            {100,  0,  0,  0,  0,  0,  0,  0,  0,  0,200,255,255,255,255,255,200,200},  //140   6
                            {100,  0,  0,  0,  0,  0,  0,  0,  0,  0,200,200,200,200,200,200,200,200},  //160   7
                            {100,250,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,100},  //180   8
                            {100,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0},  //200   9
                            {100,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0},  //220  10
                            {100,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0},  //240  11
                            {100,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0},  //260  12
                            {222,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,100},  //280  13
                            {255,255,  0,  0,  0,200,200,200,200,200,200,200,  0,  0,  0,  0,  0,100},  //300  14
                            {255,255,  0,  0,  0,200,200,200,200,200,200,200,  0,  0,255,255,255,255},  //320  15
                            {255,255,  0,  0,  0,200,200,200,200,200,200,200,  0,  0,255,255,255,255},  //340  16
                            {100,  0,  0,  0,  0,200,200,255,255,255,200,200,  0,  0,255,255,255,255},  //360  17
                            { 50,  0,  0,  0,  0,200,200,255,255,255,200,200,  0,  0,255,255,255,255},  //380  18
                            {  0,  0,  0,  0,  0,200,200,255,255,255,200,200,  0,  0,255,255,255,255},  //400  19
                            {  0,  0,  0,  0,  0,200,200,255,255,255,200,200, 80,  0,255,255,255,255},  //420  20
                            {  0,  0,  0,  0,  0,200,200,255,255,255,200,200,  0,  0,255,255,255,255},  //440  21
                            {  0,  0,  0,  0,  0,200,200,255,255,255,200,200,  0,  0,255,255,255,255},  //460  22
                            {  0,  0,  0,  0,  0,200,200,200,200,200,200,200,  0,  0,255,255,255,255},  //480  23
                            {  0,  0,  0,  0,  0,200,200,200,200,200,200,200,  0,  0,255,255,255,255},  //500  24
                            {  0,  0,  0,  0,  0,200,200,200,200,200,200,200,  0,  0,255,255,255,255},  //520  25
                            {  0,  0,  0,  0,  0,  0,  0,  0, 60,  0,  0,  0,  0,  0,  0,  0,  0,255},  //540  26
                            {  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, 70,  0,  0,255,255},  //560  27
                            {  0,  0,  0,  0,  0,255,255,255,255,255,255,255,  0,  0,  0,  0,255,255},  //580  28
                            {  0,  0,  0,  0,  0,255,255,255,255,255,255,255,  0,255,255,  0,255,255},  //600  29
                            {  0,  0,  0,  0,  0,255,255,255,255,255,255,255,  0,255,255,  0,255,255},  //620  30
                            {  0,  0,  0,  0,  0,255,255,255,255,255,255,255,  0,  0,  0,  0,255,255},  //640  31
                            {100,  0,  0,  0,  0,255,255,255,255,255,255,255,255,255,255,255,255,255}}; //660  32

// 40 = wayPoint1    = LivingRoomMap[4][0]
// 50 = wayPoint2    = LivingRoomMap[18][0]
// 60 = loveSeat     = LivingRoomMap[26][8]
// 70 = computer     = LivingRoomMap[27][13]
// 80 = couch        = LivingRoomMap[20][12]
// 90 = 
//100 = nearWall
//200 = carpet
//210 = toyDrop      = LivingRoomMap[1][10]
//222 = charger      = LivingRoomMap[13][0]
//250 = flowerPot    = LivingRoomMap[8][1]
//255 = fixedObject
*/

byte I2C[5]={0,0,0,0,0}; 
#define Address   0
#define Command   1
#define Value1    2
#define Value2    3

#define servoCenter 90
#define MapXcenter  65
#define MapYcenter  65

#define SR 1 //Sharp Short Range sensor
#define MR 2 //Sharp Medium Range sensor
#define LR 3 //Sharp Long Range sensor

//LCD
Nokia_lcd lcd=Nokia_lcd();
//Camera
AVRcam myCam=AVRcam();

Servo Pan;
Servo Tilt;

//-----------------------------------------------------------------------------


void setup() { 
  //pinMode(CS, OUTPUT);
  //digitalWrite(CS, HIGH);
  //pinMode(CLK, OUTPUT);
  //digitalWrite(CLK, HIGH);
  //pinMode(SDA, OUTPUT);
  //digitalWrite(SDA, HIGH);
  //pinMode(RESET, OUTPUT);
  //digitalWrite(RESET, HIGH);

  /*pinMode(Sw1Pin, INPUT); 
  digitalWrite(Sw1Pin, HIGH);//turn on pull-up resistor
  pinMode(Sw2Pin, INPUT); 
  digitalWrite(Sw2Pin, HIGH);//turn on pull-up resistor
  pinMode(RedPin, OUTPUT); 
  digitalWrite(RedPin, HIGH);
  pinMode(GreenPin, OUTPUT); 
  digitalWrite(GreenPin, HIGH);
  pinMode(BluePin, OUTPUT); 
  digitalWrite(BluePin, HIGH);*/
  pinMode(PingPin, OUTPUT); 
  digitalWrite(PingPin, LOW);

  Pan.attach(PanPin);
  Pan.write(servoCenter); // 0 - 90 - 180
  Tilt.attach(TiltPin);
  Tilt.write(servoCenter); // 0 - 90 - 180

  //nokiaLcd.lcd_init();
  DDRB=0x2F;
  lcd.cLCD_Init();
  lcd.cLCD_Box(0,0,131,131,FILL,LIGHTBLUE);
  delay(100);
  //lcd.cLCD_String("Ready", 50, 115, BLACK,WHITE);
  delay(100);
  
  //AVRcam initialisation - initializes the hardware serial at 115200 bps
  myCam.Init(1, 1, 1);

  //Serial.begin (9600);
  //Serial.println("start");

  Wire.begin(Mapper);     // join i2c bus
  Wire.onReceive(I2CreceiveEvent); // register event

} 
  //int d=0;


void loop(){
    //digitalWrite(BluePin, LOW);
  //Process_commands();
  //SoftwareServo::refresh();
    //digitalWrite(BluePin, HIGH);
  //delay(50);
  ScanMap();
  //CheckSensors();
  //isFire();
  //delay(5000);
  //FireFight();
  //testSharp();
  //Read_Camera();
  //Read_Compass_Sensor();
  //delay(500);
}  

void testSharp(){
  lcd.cLCD_Box(60,80,90,110,FILL,LIGHTBLUE);
  byte pos=63;
  int Lval=0;
  //int Cval=0;
  //int Rval=0;
  Lval = analogRead(SleftPin);
  //Lval=Read_Sharp_Sensor(LR, SleftPin);
        itoa(Lval, byteString, 10);
        strcpy(text, byteString);
        lcd.cLCD_String(text, 63, 90, BLUE,BLACK);
  delay(500);
  //Cval=Read_Sharp_Sensor(SR, SscanPin);
  //Rval=Read_Sharp_Sensor(LR, SrightPin);
        //itoa(Rval, byteString, 10);
        //strcpy(text, byteString);
        //lcd.cLCD_String(text, (pos+30), 115, YELLOW,BLACK);
  /*Serial.print("Left: ");
  Serial.print(Lval, DEC);
  Serial.print("  Center: ");
  Serial.print(Cval, DEC);
  Serial.print("   Right: ");
  Serial.println(Rval, DEC);*/
}

void Process_commands() {
    switch (I2C[Command]) {
      case Fire:
        FireFight();
        I2C[Command]=0;
        break;
      case Scan:
        ScanMap();
        I2C[Command]=0;
        break;
      case Dance:
        Dancing();
        I2C[Command]=0;
        break;
      case Done:
        odometer = 0;
        I2C[Command]=0;
        break;
    }
}

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

void Dancing()
{
  for (int i = 0; i < 3; i++) {
    //I2C_transmit(ServoController,Lshoulder,center,0);
    //I2C_transmit(ServoController,Rshoulder,center,1;
    delay(100);
    //I2C_transmit(PanHead,left,1);
    delay(100);
    //I2C_transmit(PanHead,center,0);
    //I2C_transmit(ServoController,Lshoulder,down,0);
    //I2C_transmit(ServoController,Rshoulder,up,1;
    I2C_transmit(MotorController,Left,0,200);
    delay(1000);
    I2C_transmit(MotorController,StopAll,0,0);
    delay(100);
    //I2C_transmit(PanHead,right,1);
    delay(100);
    //I2C_transmit(PanHead,center,0);
    //I2C_transmit(ServoController,Lshoulder,center,0);
    //I2C_transmit(ServoController,Rshoulder,center,1;
    I2C_transmit(MotorController,Right,0,200);
    delay(1000);
    I2C_transmit(MotorController,StopAll,0,0);
    delay(100);
    //I2C_transmit(PanHead,right,0);
    delay(10);
    I2C_transmit(MotorController,Right,0,200);
    //Lshoulder.write(up);
    //Rshoulder.write(down);
    I2C_transmit(MotorController,StopAll,0,0);
    delay(10);
    //Pan.write(center);
    delay(10);
    I2C_transmit(MotorController,Left,0,200);
    //Lshoulder.write(center);
    //Rshoulder.write(center);
    I2C_transmit(MotorController,StopAll,0,0);
    //Lshoulder.write(down);
    //Rshoulder.write(down);
    I2C_transmit(MotorController,Forward,0,200);
    I2C_transmit(MotorController,StopAll,0,0);
  }
}

byte Find_Hot_Object() {
  Read_Thermopile_Array();
  byte cluster = 0;
  //byte hotObject = temperature[0];  // ambient temperature
  for(byte i = 1; i < 9; i++){  
    if (temperature[i] > hotObject) {
      hotObject = temperature[i]; // store the highest temperature
      cluster = i; // store the location of the hottest spot
    }
  }
  //Serial.print("Hot Object: ");
  //Serial.print(hotObject, DEC);
  //Serial.print("  ");
  //Serial.print("in cluster ");
  //Serial.print(cluster, DEC);
  //Serial.println("   ");
  if (hotObject>28) return cluster; else return 0;
}

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++



//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

void I2CreceiveEvent(int i)
{
    digitalWrite(RedPin, LOW);
    //Serial.println("I2C communication received:");
    while(0 < Wire.available()) // loop through all
    {
      I2C[i] = Wire.receive(); // receive bytes
      //Serial.print(i, DEC);
      //Serial.print("  ");
      //Serial.println(I2C[i], DEC);
      i = i-1;
    }
    digitalWrite(RedPin, HIGH);
    if (I2C[Command]==Done) odometer=0;
}

void I2C_transmit(byte Addr, byte Cmnd, byte Val1, byte Val2){
  Wire.beginTransmission(Addr); // transmit to slave device
  Wire.send(Val2);         // sends one byte  
  Wire.send(Val1);         // sends one byte  
  Wire.send(Cmnd);        // sends one byte
  Wire.endTransmission();    // stop transmitting
}

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

